<?php


if (!defined('__SIMPLERSSWRITER_INC__')) {
    define('__SIMPLERSSWRITER_INC__', 1);	

    define("RSS_CRLF","\n");

    
    //
    // Global functions
    //
    function EncodeRssText($strText)	{
    	$strText = EscapeText($strText);
    	return utf8_encode($strText);
    }
    
    function EscapeText($strText) 		{
		
    	$iPos=0;
		$iLength=strlen($strText);
		$strEscaped = "";

		for(;$iPos < $iLength;)		{
			$cChar = substr($strText,$iPos,1);
			$iCode = ord($cChar);
			switch($iCode)  	{
				case 34:
					$cChar="&quot;"; break;
				case 38:
					$cChar="&amp;"; break;
				case 39:
					$cChar="&apos;"; break;
				case 60:
					$cChar="&lt;"; break;
				case 62:
					$cChar="&gt;"; break;
				default:
					if($iCode<32)
						$iChar=("&#".strval($iCode).";");
					break;
			}
			$strEscaped.=$cChar;
			$iPos++;
		}
		return $strEscaped;
	}
    
    
    //
    // RSS Writer
    //    
	class SimpleRssWriter	{	
	
		var $oRssChannel = NULL;
		
		var $strRssVersion = "2.0";
		var $strXMLVersion = "1.0";
		var $strEncoding = "UTF-8";

		function SimpleRssWriter(&$oChannel)	{
			$this->oRssChannel = $oChannel;
		}
		
		function WriteXML()		{
			
			$strXML = "<?xml version=\"".$this->strXMLVersion."\" encoding=\"".$this->strEncoding."\" ?>".RSS_CRLF.
				"<rss version=\"".$this->strRssVersion."\">".RSS_CRLF.
				$this->oRssChannel->WriteXML().RSS_CRLF.
				"</rss>".RSS_CRLF;
			return $strXML;			
		}
		
	}
	
	class RssChannel	{
		
		// Items
		var $arrRssItems = array();
		
		// Mandatory properties
		var $strTitle = "";
		var $strLink = "";
		var $strDescription = "";
		
		// Optional properties
		var $strLanguage = "";
		var $strCopyright = "";
		var $strManagingEditor = "";
		var $strWebmaster = "";
		var $dtPubdate = NULL;
		var $strGenerator = "";
		var $iTtl = 720;
		var $strImage = "";
		
		var $strError = "";
		
		
		function RssChannel($strTitle="",$strLink="",$strDescription="")	{
			$this->strTitle = EncodeRssText($strTitle);
			$this->strLink = EncodeRssText($strLink);
			$this->strDescription = EncodeRssText($strDescription);
		}

		
		function AddItem(&$oItem)	{
			$this->arrRssItems[] = $oItem;
			return TRUE;
		}

		
		function GetError()		{
			return $this->strError;
		}
			
		
		function WriteXML()		{

			// Write channel XML
			$strXML = "<channel>".RSS_CRLF.
				(!empty($this->strTitle)?"<title>".$this->strTitle."</title>".RSS_CRLF:"").						
				(!empty($this->strLink)?"<link>".$this->strLink."</link>".RSS_CRLF:"").
				(!empty($this->strDescription)?"<description>".$this->strDescription."</description>".RSS_CRLF:"").						
				(!empty($this->strLanguage)?"<language>".$this->strLanguage."</language>".RSS_CRLF:"").						
				(!empty($this->strCopyright)?"<copyright>".$this->strCopyright."</copyright>".RSS_CRLF:"").						
				(!empty($this->strManagingEditor)?"<managingEditor>".$this->strManagingEditor."</managingEditor>".RSS_CRLF:"").						
				(!empty($this->strWebmaster)?"<webMaster>".$this->strWebmaster."</webMaster>".RSS_CRLF:"").						
				(!empty($this->dtPubdate)?"<pubDate>".date("r",$this->dtPubdate)."</pubDate>".RSS_CRLF:"").						
				(!empty($this->strGenerator)?"<generator>".$this->strGenerator."</generator>".RSS_CRLF:"").						
				(!empty($this->iTtl)?"<ttl>".$this->iTtl."</ttl>".RSS_CRLF:"");

			if(!empty($this->strImage))		{
				$strXML.= "<image>".RSS_CRLF.
					"<url>".$this->strImage."</url>".RSS_CRLF.
					"<title>".$this->strTitle."</title>".RSS_CRLF.
					"<link>".$this->strLink."</link>".RSS_CRLF.
					"</image>".RSS_CRLF;
			}
			
			// Add items
			foreach($this->arrRssItems as $oItems)	{
				$strXML.= $oItems->WriteXML();
			}
			$strXML.= "</channel>";			

			return $strXML;		
		}
		
		
		function CheckData()	{
			
			if(empty($this->strTitle) ||
				empty($this->strLink) ||
				empty($this->strDescription))	{
			
				$this->strError = "Channel Error: Mandatory fields missing.";					
				return FALSE;	
			}
			
			if(sizeof($this->arrRssItems)==0)	{
				$this->strError = "Channel Error: No items defined.";
				return FALSE;
			} else {
				foreach($this->arrRssItems as $oItem)	{
					// Check each item
					if($oItem->CheckData())		{
						$this->strError = $oItem->GetError();
						return FALSE;
					}
				}
			}
			return TRUE;
		}
	}
	
	
	class RssItem	{
		
		var $strTitle = "";
		var $strDescription = "";
		var $strLink = "";
		
		var $dtPubDate = NULL;
		var $strGuid = "";
		var $strAuthor = "";

		var $strError = "";
		
		
		// Constructor		
		function RssItem($strTitle="",$strLink="",$strDescription="")		{
			$this->strTitle = EncodeRssText($strTitle);
			$this->strLink = $strLink;
			$this->strDescription = EncodeRssText($strDescription);
			
			$this->dtPubDate = mktime();
		}
		
		
		function GetError()	{
			return $this->strError;
		}
		
		
		function WriteXML()		{
			
			$strXML = "<item>".RSS_CRLF.
				(!empty($this->strTitle)?"<title>".$this->strTitle."</title>".RSS_CRLF:"").
				(!empty($this->strLink)?"<link>".$this->strLink."</link>".RSS_CRLF.
					"<guid>".$this->strLink."</guid>".RSS_CRLF:"").
				(!empty($this->strDescription)?"<description>".$this->strDescription."</description>".RSS_CRLF:"").
				(!is_null($this->dtPubDate)?"<pubDate>".date("r",$this->dtPubDate)."</pubDate>".RSS_CRLF:"").
				(!empty($this->strGuid)?"<guid>".$this->strGuid."</guid>".RSS_CRLF:"").
				(!empty($this->strAuthor)?"<guid>".$this->strAuthor."</guid>".RSS_CRLF:"").
				"</item>";
			return $strXML;						
		}
		
		
		function CheckData()	{
			
			if(empty($this->strTitle) && empty($this->strDescription))	{
				$this->strError = "Item Error: Mandatory fields missing. Set title and/or description.";	
				return FALSE;				
			}
			return TRUE;
		}
	}

}

?>